@@ -5,8 +5,8 @@ if sys.version_info[0] < 3:
 else:
     from urllib.parse import urlparse
 
-import awkward
-import uproot
+import awkward0 as awkward
+import uproot3 as uproot
 
 input_filenames_argument_name = 'input_filenames'
 tree_name_argument_name = 'tree_name'
@@ -118,7 +118,7 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         if (id in ('True', 'False', 'None')
            or id in self._id_scopes
            or id in self.get_globals()
-           or id in ('abs', 'all', 'any', 'len', 'max', 'min')):
+           or id in ('abs', 'all', 'any', 'len', 'max', 'min', 'sum')):
             return id
         else:
             raise NameError('Unknown id: ' + id)
@@ -196,13 +196,15 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         return node
 
     def visit_ExtSlice(self, node):
-        node.rep = ', '.join(self.get_rep(dimension) for dimension in node.dims)
+        node.rep = '(' + ', '.join(self.get_rep(dimension) for dimension in node.dims) + ')'
         return node
 
     def visit_Subscript(self, node):
         value_rep = self.get_rep(node.value)
         slice_rep = self.get_rep(node.slice)
-        node.rep = value_rep + '[' + slice_rep + ']'
+        node.rep = ('(' + value_rep + '[' + value_rep + '.columns[' + slice_rep + ']]'
+                    + ' if isinstance(' + value_rep + ', awkward.Table)'
+                    + ' and ' + value_rep + '.istuple else ' + value_rep + '[' + slice_rep + '])')
         return node
 
     def visit_Attribute(self, node):
@@ -240,7 +242,6 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
             if len(node.args) > 2:
                 raise TypeError('EventDataset() should have no more than two arguments, found '
                                 + str(len(node.args)))
-
             if len(node.args) >= 1:
                 if hasattr(node.args[0], 'elts'):
                     urls = node.args[0].elts
@@ -252,7 +253,6 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                               + 'else ' + repr(paths))
             else:
                 source_rep = input_filenames_argument_name
-
             if len(node.args) >= 2:
                 local_tree_name_rep = self.get_rep(node.args[1])
             else:
@@ -260,13 +260,13 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                                        + "key_array[key_array[:, 1] == 'TTree'][:, 0])("
                                        + 'awkward.Table(uproot.open(input_files[0]).classnames())'
                                        + '.unzip()[0])[0]')
-            tree_name_rep = (tree_name_argument_name + ' '
+            tree_name_rep = ('(' + tree_name_argument_name + ' '
                              + 'if ' + tree_name_argument_name + ' is not None '
-                             + 'else ' + local_tree_name_rep)
-
+                             + 'else ' + local_tree_name_rep + ')')
             node.rep = ('(lambda input_files: '
                         + 'uproot.lazyarrays(input_files, '
-                        + tree_name_rep
+                        + "logging.getLogger(__name__).info('Using treename=' + repr("
+                        + tree_name_rep + ')) or ' + tree_name_rep
                         + '))(' + source_rep + ')')
         else:
             func_rep = self.get_rep(node.func)
@@ -285,17 +285,35 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
             node.selector.body = ast.Call(func=ast.Attribute(value=ast.Name(id='awkward'),
                                                              attr='Table'),
                                           args=node.selector.body.elts)
-        if type(node.selector.body) is ast.Dict:
+        elif type(node.selector.body) is ast.Dict:
             node.selector.body = ast.Call(func=ast.Attribute(value=ast.Name(id='awkward'),
                                                              attr='Table'),
                                           args=[node.selector.body])
-        call_node = self.visit(ast.Call(func=node.selector, args=[node.source]))
+        call_node = ast.Call(func=node.selector, args=[node.source])
         node.rep = self.get_rep(call_node)
         return node
 
     def visit_SelectMany(self, node):
-        node = self.visit_Select(node)
-        call_node = self.visit(ast.Call(func=ast.Attribute(value=node, attr='flatten'), args=[]))
+        if type(node.selector) is not ast.Lambda:
+            raise TypeError('Argument to SelectMany() must be a lambda function, found '
+                            + node.selector)
+        if len(node.selector.args.args) != 1:
+            raise TypeError('Lambda function in SelectMany() must have exactly one argument, '
+                            'found ' + len(node.selector.args.args))
+        if type(node.selector.body) in (ast.List, ast.Tuple):
+            node.selector.body.elts = [ast.Call(func=ast.Attribute(value=element,
+                                                                   attr='flatten'),
+                                                args=[]) for element in node.selector.body.elts]
+        elif type(node.selector.body) is ast.Dict:
+            node.selector.body.values = [ast.Call(func=ast.Attribute(value=dict_value,
+                                                                     attr='flatten'),
+                                                  args=[])
+                                         for dict_value in node.selector.body.values]
+        else:
+            node.selector.body = ast.Call(func=ast.Attribute(value=node.selector.body,
+                                                             attr='flatten'),
+                                          args=[])
+        call_node = self.visit_Select(node)
         node.rep = self.get_rep(call_node)
         return node
 
@@ -306,14 +324,15 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         if len(node.predicate.args.args) != 1:
             raise TypeError('Lambda function in Where() must have exactly one argument, found '
                             + len(node.predicate.args.args))
-
         if sys.version_info[0] < 3:
             subscriptable = node.predicate.args.args[0].id
         else:
             subscriptable = node.predicate.args.args[0].arg
-
-        node.predicate.body = ast.Subscript(value=ast.Name(id=subscriptable),
-                                            slice=ast.Index(node.predicate.body))
+        if sys.version_info[0] < 3 or (sys.version_info[0] == 3 and sys.version_info[1] < 9):
+            slice_node = ast.Index(node.predicate.body)
+        else:
+            slice_node = node.predicate.body
+        node.predicate.body = ast.Subscript(value=ast.Name(id=subscriptable), slice=slice_node)
         call_node = self.visit(ast.Call(func=node.predicate, args=[node.source]))
         node.rep = self.get_rep(call_node)
         return node
